unit delmenu;
{ TDelSysMenuItems verion 1.0 copyright  1996 by Alexander Meeder

  This non-visual component is build round the Windows 'DeleteMenu' API. When you set
  the form-property 'bordericons' without 'Minimize' and/or 'Maximize' the SystemMenu
  items were dimmed, not deleted. With this component you can do it easily by setting
  the desired SysMenuItem to true.

  properties (runtime only):
  * SysMenuItem (smClose, smMaximize, smMinimize etc.)

  If you have suggestions about TDelSysMenuItems or make changes, pleas mail me: ameeder@dds.nl
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  EDelSysMenuItemError = class(Exception);
  TSysMenuItem = (smClose, smMaximize, smMinimize, smMove, smNextWindow, smRestore, smSize);
  TSysmenuItems = set of TSysMenuItem;

  TDelSysMenuItems = class(TComponent)
  private
    FSysMenuItems: TSysMenuItems;
  protected
    hSysMenu: HMENU;
    IsChildWindow: boolean;
    BorderIcons: TBorderIcons;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property SysMenuItems: TSysMenuItems read FSysMenuItems write FSysMenuItems;
  end;

procedure Register;

implementation

procedure TDelSysMenuItems.Loaded;
begin
  if not (csDesigning in ComponentState) then
  begin
    if (smClose in FSysMenuItems) and (biSystemMenu in BorderIcons) then
    begin
      if not DeleteMenu(hSysMenu, SC_CLOSE, MF_BYCOMMAND) then
         raise EDelSysMenuItemError.Create('Can''t delete item from systemmenu!');
    end;
    if (smMaximize in FSysMenuItems) and (biSystemMenu in BorderIcons) and not (biMaximize in BorderIcons) then
    begin
      if not DeleteMenu(hSysMenu, SC_MAXIMIZE, MF_BYCOMMAND) then
         raise EDelSysMenuItemError.Create('Can''t delete item from systemmenu!');
    end;
    if (smMinimize in FSysMenuItems) and (biSystemMenu in BorderIcons) and not (biMinimize in BorderIcons) then
    begin
      if not DeleteMenu(hSysMenu, SC_MINIMIZE, MF_BYCOMMAND) then
         raise EDelSysMenuItemError.Create('Can''t delete item from systemmenu!');
    end;
    if (smMove in FSysMenuItems) and (biSystemMenu in BorderIcons) then
    begin
      if not DeleteMenu(hSysMenu, SC_MOVE, MF_BYCOMMAND) then
         raise EDelSysMenuItemError.Create('Can''t delete item from systemmenu!');
    end;
    if (smNextWindow in FSysMenuItems) and (IsChildWindow) and (biSystemMenu in BorderIcons) then
    begin
      if not DeleteMenu(hSysMenu, SC_NEXTWINDOW, MF_BYCOMMAND) then
         raise EDelSysMenuItemError.Create('Can''t delete item from systemmenu!');
    end;
    if (smRestore in FSysMenuItems) and (biSystemMenu in BorderIcons) then
    begin
      if not DeleteMenu(hSysMenu, SC_RESTORE, MF_BYCOMMAND) then
         raise EDelSysMenuItemError.Create('Can''t delete item from systemmenu!');
    end;
    if (smSize in FSysMenuItems) and (biSystemMenu in BorderIcons) then
    begin
      if not DeleteMenu(hSysMenu, SC_SIZE, MF_BYCOMMAND) then
         raise EDelSysMenuItemError.Create('Can''t delete item from systemmenu!');
    end;
  end;
end;

constructor TDelSysMenuItems.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  hSysMenu := GetSystemMenu((AOwner as TForm).Handle, false);
  if (AOwner as TForm).FormStyle = fsMDIChild then IsChildWindow := true else IsChildWindow := false;
  BorderIcons := (AOwner as TForm).BorderIcons;
  if not (biMinimize in BorderIcons) then FSysMenuItems := [smMinimize];
  if not (biMaximize in BorderIcons) then FSysMenuItems := FSysMenuItems + [smMaximize];
end;

destructor TDelSysMenuItems.Destroy;
begin
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TDelSysMenuItems]);
end;

end.
