{ TKeyState verion 1.0 copyright  1996 by Alexander Meeder

  This non-visual component is build round Windows 'GetKeyState' API (idea from answer of
  Brad Stowers in Delphi news-group).

  properties:
  CapsLock (true/false)
  NumLock (    "     )
  ScrollLock (   "   )
  Interval (set update-interval)

  event:
  update (you can write your own code, for instance update your statusbar
         procedure TForm1.update(Sender: TObject);
         begin
           if KeyStatus1.CapsLock then StatusBar1.Panels.Items[0].Text := 'CAPS'
           else StatusBar1.Panels.Items[0].Text := '';
           ....
         end;

  If you have suggestions about TKeyStatus or make changes, pleas mail me: ameeder@dds.nl
}

unit keystate;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ExtCtrls;

type
  TKeyState = (ksOn, ksOff);
  TInterval = 0..10000;
  TUpdate = procedure(Sender: TObject) of object;

  TKeyStatus = class(TComponent)
  private
    FEnabled: boolean;
    FCapsLock: TKeyState;
    FNumLock: TKeyState;
    FScrollLock: TKeyState;
    FTimer : TTimer;
    FOnUpdate: TUpdate;
    FInterval : TInterval;

    procedure SetInterval(value: TInterval);
    procedure SetEnabled(value: boolean);
  protected
    DummyKeyState: TKeyState;
    procedure Update;
    procedure TimerTimer(Sender: TObject);
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled: boolean read FEnabled write SetEnabled default true;
    property CapsLock: TKeyState read FCapsLock write DummyKeyState;
    property NumLock: TKeyState read FNumLock write DummyKeyState;
    property ScrollLock: TKeyState read FScrollLock write DummyKeyState;
    property Interval: TInterval read FInterval write SetInterval default 500;
    property OnUpdate: TUpdate read FOnUpdate write FOnUpdate;
  end;

procedure Register;

implementation

procedure TKeyStatus.Update();
begin
  if assigned(FOnUpdate) then FOnUpdate(self);
end;

procedure TKeyStatus.SetInterval(value: TInterval);
begin
  if Value <> FInterval then
  begin
    FInterval := Value;
  end;
end;

procedure TKeyStatus.SetEnabled(value: boolean);
begin
  if (value <> FEnabled) then
  begin
    FEnabled := value;
    if not (csDesigning in Componentstate) then
    begin
      FTimer.Enabled := FEnabled;
    end;
  end;
end;

procedure TKeyStatus.TimerTimer(Sender: TObject);
begin
  if Odd(GetKeyState(VK_CAPITAL)) then
     FCapsLock := ksOn
  else
     FCapsLock := ksOff;
  if Odd(getKeyState(VK_NUMLOCK)) then
     FNumLock := ksOn
  else
     FNumLock := ksOff;
  if Odd(GetKeyState(VK_SCROLL)) then
     FScrollLock := ksOn
  else
     FScrollLock := ksOff;
  Update;
end;

procedure TKeyStatus.Loaded;
begin
  if not (csDesigning in Componentstate) then
  begin
    FTimer.Enabled := FEnabled;
  end;
end;

constructor TKeyStatus.Create(AOwner: TComponent);
begin
  inherited;
  FEnabled := true;
  FInterval := 500;
  if not (csDesigning in ComponentState) then
  begin
    FTimer := TTimer.Create(self);
    FTimer.Interval := FInterval;
    FTimer.OnTimer := TimerTimer;
    FTimer.Enabled := FEnabled;
  end;
end;

destructor TKeyStatus.Destroy;
begin
  FTimer.Free;
  inherited;
end;

procedure Register;
begin
  RegisterComponents('AM Software', [TKeyStatus]);
end;

end.
