{ TPosition version 1.0 copyright  1996 by Alexander Meeder

  With this component you can save the position and/or dimension of the form that owns this control.

  properties:
  * IniFileDirectory -> Directory where to store the inifile, Windows or Program, default = program
  * IniFileName -> Name of inifile to save Left, Top etc.
  * SaveDimension -> true/false, saves Width and Height of owner-form
  * SavePosition -> true/false, saves Left and Top of owner-form

  method:
  * SaveSettings -> Call this method just before quiting your application (Form1.OnClose or
                    Form1.OnDestroy)
}
unit position;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, IniFiles;

type
  TIniFileDirectory = (ProgramDir, WindowsDir);

  TPosition = class(TComponent)
  private
    FIniFileDirectory: TIniFileDirectory;
    FIniFileName: string;
    FSaveDimension: boolean;
    FSavePosition: boolean;

    procedure SetIniFileDirectory(value: TIniFileDirectory);
    procedure SetIniFileName(value: string);
    procedure SetSaveDimension(value: boolean);
    procedure SetSavePosition(value: boolean);
  protected
    procedure Loaded; override;
  public
    procedure SaveSettings;
    constructor Create(AOwner: TComponent); override;
  published
    property IniFileDirectory: TIniFileDirectory read FIniFileDirectory write SetIniFileDirectory default ProgramDir;
    property IniFileName: string read FIniFileName write SetIniFileName;
    property SaveDimension: boolean read FSaveDimension write SetSaveDimension default true;
    property SavePosition: boolean read FSavePosition write SetSavePosition default true;
  end;

procedure Register;

implementation

procedure TPosition.SaveSettings;
var
  IniFile: TIniFile;
begin
  if not (csDesigning in Componentstate) then
    begin
    case FIniFileDirectory of
      ProgramDir : IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + FIniFileName);
      WindowsDir  : IniFile := TIniFile.Create(FIniFileName);
    end;
    try
      if FSaveDimension then
      begin
        IniFile.WriteInteger('Values', 'Width', (Owner as TForm).Width);
        IniFile.WriteInteger('Values', 'Height', (Owner as TForm).Height);
      end;
      if FSavePosition then
      begin
        IniFile.WriteInteger('Values', 'Left', (Owner as TForm).Left);
        IniFile.WriteInteger('Values', 'Top', (Owner as TForm).Top);
      end;
    finally
      IniFile.Free;
    end;
  end;
end;

procedure TPosition.SetIniFileDirectory(value: TIniFileDirectory);
begin
  if value <> FIniFileDirectory then
  begin
    FIniFileDirectory := value;
  end;
end;

procedure TPosition.SetIniFileName(value: string);
begin
  if (value <> FIniFileName) and (ExtractFileExt(value) = '.ini') and (Length(value) <= 12) then
  begin
    FIniFileName := LowerCase(value);
  end;
end;

procedure TPosition.SetSaveDimension(value: boolean);
begin
  if value <> FSaveDimension then
  begin
    FSaveDimension := value;
  end;
end;

procedure TPosition.SetSavePosition(value: boolean);
begin
  if value <> FSavePosition then
  begin
    FSavePosition := value;
  end;
end;

procedure TPosition.Loaded;
var
  IniFile: TIniFile;
begin
  inherited Loaded;
  if not (csDesigning in Componentstate) then
  begin
    case FIniFileDirectory of
       ProgramDir : IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + FIniFileName);
       WindowsDir  : IniFile := TIniFile.Create(FIniFileName);
    end;
    try
      if FSaveDimension then
      begin
        (Owner as TForm).Width := IniFile.ReadInteger('Values', 'Width', (Owner as TForm).Width);
        (Owner as TForm).Height := IniFile.ReadInteger('Values', 'Height', (Owner as TForm).Height);
      end;
      if FSavePosition then
      begin
        (Owner as TForm).Left := IniFile.ReadInteger('Values', 'Left', (Screen.Width - (Owner as TForm).Width) div 2);
        (Owner as TForm).Top := IniFile.ReadInteger('Values', 'Top', (Screen.Height - (Owner as TForm).Height) div 2);
      end;
    finally
      IniFile.Free;
    end;
  end;
end;

constructor TPosition.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIniFileDirectory := ProgramDir;
  FIniFileName := (Owner as TForm).Name + '.ini';
  FSaveDimension := true;
  FSavePosition := true;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TPosition]);
end;

end.
