unit scrollbl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus;

type
  TInterval = 1..1000;
  TScrollBy = 1..10;
  TScrollDirection = (LeftToRight, RightToLeft, TopToBottom, BottomToTop);

  TScrollLabel = class(TCustomLabel)
  private
    FCaption: TCaption;
    FInterval: TInterval;
    FScrollBy: TScrollBy;
    FScrollDirection: TScrollDirection;
    FTimer: TTimer;

    function GetActive: boolean;
    procedure SetActive(Value: boolean);
    procedure SetInterval(Value: TInterval);
    procedure SetScrollBy(Value: TScrollBy);
    procedure SetScrollDirection(Value: TScrollDirection);
    procedure SetText(Value: TCaption);
  protected
    procedure UpdateTimer(Sender: TObject);
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Active: boolean read GetActive write SetActive default true;
    property Caption: TCaption read FCaption write SetText;
    property Interval: TInterval read FInterval write SetInterval default 100;
    property ScrollBy: TScrollBy read FScrollBy write SetScrollBy default 1;
    property ScrollDirection: TScrollDirection read FScrollDirection write SetScrollDirection;
    property Color;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
  end;

procedure Register;

implementation

var LastPoint: TPoint;
    CaptionWidth, CaptionHeight: integer;

function TScrollLabel.GetActive: boolean;
begin
  Result := FTimer.Enabled;
end;

procedure TScrollLabel.SetActive(Value: boolean);
begin
    FTimer.Enabled := Value;
end;

procedure TScrollLabel.SetInterval(Value: TInterval);
begin
  if (FInterval <> Value) and (Value >= 1) and (Value <= 1000) then
  begin
    FInterval := Value;
    FTimer.Interval := FInterval;
  end;
end;

procedure TScrollLabel.SetScrollBy(Value: TScrollBy);
begin
  if Value <> FScrollBy then
  begin
    FScrollBy := Value;
  end;
end;

procedure TScrollLabel.SetScrollDirection(Value: TScrollDirection);
begin
  if Value <> FScrollDirection then
  begin
    FScrollDirection := Value;
    Canvas.Font := Self.Font;
    case FScrollDirection of
       RightToLeft : begin
                       if (csDesigning in ComponentState) then
                          ClientHeight := Canvas.TextHeight(Caption);
                       LastPoint.X := Self.ClientWidth;
                       LastPoint.Y := 1;
                     end;
       LeftToRight : begin
                       if (csDesigning in ComponentState) then
                       ClientHeight := Canvas.TextHeight(Caption);
                       LastPoint.X := - CaptionWidth;
                       LastPoint.Y := 1;
                     end;
       TopToBottom : begin
                       if (csDesigning in ComponentState) then
                       ClientWidth := Canvas.TextWidth(Caption) + 2;
                       LastPoint.X := 1;
                       LastPoint.Y := - CaptionHeight;
                     end;
       BottomToTop : begin
                       if (csDesigning in ComponentState) then
                       ClientWidth := Canvas.TextWidth(Caption) + 2;
                       LastPoint.X := 1;
                       LastPoint.Y := Self.ClientHeight;
                     end;
    end;
    invalidate;
  end;
end;

procedure TScrollLabel.SetText(Value: TCaption);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
  end;
end;

procedure TScrollLabel.UpdateTimer(Sender: TObject);
begin
  invalidate;
end;

procedure TScrollLabel.Paint;
begin
  CaptionWidth := Canvas.TextWidth(Caption);
  CaptionHeight := Canvas.TextHeight(Caption);

  Canvas.Font := Self.Font;
  Canvas.Brush.Color := Canvas.Font.Color;
  Canvas.Brush.Style := bsClear;

  case ScrollDirection of
       RightToLeft : begin
                       LastPoint.Y := 1;
                       dec(LastPoint.X, FScrollBy);
                       if LastPoint.X <= - CaptionWidth then LastPoint.X := Self.ClientWidth;
                     end;
       LeftToRight : begin
                       LastPoint.Y := 1;
                       inc(LastPoint.X, FScrollBy);
                       if LastPoint.X >= Self.ClientWidth then LastPoint.X := - CaptionWidth;
                     end;
       TopToBottom : begin
                       LastPoint.X := 1;
                       inc(LastPoint.Y, FScrollBy);
                       if LastPoint.Y >= Self.ClientHeight then LastPoint.Y := - CaptionHeight;
                     end;
       BottomToTop : begin
                       LastPoint.X := 1;
                       dec(LastPoint.Y, FScrollBy);
                       if LastPoint.Y <= - CaptionHeight then LastPoint.Y := Self.ClientHeight;
                     end;
  end;


  Canvas.TextOut(LastPoint.X, LastPoint.Y, Caption);
end;

constructor TScrollLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FCaption := 'ScrollLabel';
  FScrollBy := 1;
  FInterval := 100;

  Transparent := true;

  RegisterClasses([TTimer]);
  FTimer := TTimer.Create(AOwner);
  FTimer.OnTimer := UpdateTimer;
  FTimer.Interval := FInterval;
end;

destructor TScrollLabel.Destroy;
begin
  FTimer.Free;
  inherited;
end;

procedure Register;
begin
  RegisterComponents('Additional', [TScrollLabel]);
end;

end.
